@@ -3,6 +3,7 @@ import os
 import warnings
 from contextlib import contextmanager
 from pathlib import Path
+from types import SimpleNamespace
 from typing import Generator, Optional
 from unittest.mock import Mock, patch
 
@@ -16,7 +17,14 @@ from huggingface_hub.cli.download import download
 from huggingface_hub.cli.hf import app
 from huggingface_hub.cli.upload import _resolve_upload_paths, upload
 from huggingface_hub.errors import RevisionNotFoundError
-from huggingface_hub.utils import CachedRepoInfo, CachedRevisionInfo, SoftTemporaryDirectory
+from huggingface_hub.utils import (
+    CachedFileInfo,
+    CachedRepoInfo,
+    CachedRevisionInfo,
+    HFCacheInfo,
+    SoftTemporaryDirectory,
+)
+from huggingface_hub.utils._verification import FolderVerification
 
 from .testing_utils import DUMMY_MODEL_ID
 
@@ -118,6 +126,31 @@ class TestCacheCommand:
         assert result.exit_code == 0
         assert result.stdout.strip() == revision.commit_hash
 
+    def test_ls_with_sort(self, runner: CliRunner) -> None:
+        repo1 = _make_repo("user/model1", revisions=[_make_revision("d" * 40)])
+        repo2 = _make_repo("user/model2", revisions=[_make_revision("e" * 40)])
+        repo3 = _make_repo("user/model3", revisions=[_make_revision("f" * 40)])
+        entries = [(repo1, None), (repo2, None), (repo3, None)]
+        repo_refs_map = {repo1: frozenset(), repo2: frozenset(), repo3: frozenset()}
+
+        with (
+            patch("huggingface_hub.cli.cache.scan_cache_dir"),
+            patch(
+                "huggingface_hub.cli.cache.collect_cache_entries",
+                return_value=(entries, repo_refs_map),
+            ),
+        ):
+            result = runner.invoke(app, ["cache", "ls", "--sort", "name:desc", "--limit", "2"])
+
+        assert result.exit_code == 0
+        stdout = result.stdout
+
+        # Check alphabetical order
+        assert stdout.index("model3") < stdout.index("model2")  # descending order
+
+        # Check limit of 2 entries
+        assert "model1" not in stdout
+
     def test_rm_revision_executes_strategy(self, runner: CliRunner) -> None:
         revision = _make_revision("c" * 40)
         repo = _make_repo("user/model", revisions=[revision])
@@ -207,6 +240,133 @@ class TestCacheCommand:
         strategy.execute.assert_not_called()
         print_mock.assert_called_once()
 
+    def test_verify_success(self, runner: CliRunner) -> None:
+        repo_id = "user/model"
+        verified_path = Path("/tmp/cache/user/model")
+        result_obj = FolderVerification(
+            revision="main",
+            checked_count=1,
+            mismatches=[],
+            missing_paths=[],
+            extra_paths=[],
+            verified_path=verified_path,
+        )
+
+        with patch("huggingface_hub.cli.cache.get_hf_api") as get_api_mock:
+            api = get_api_mock.return_value
+            api.verify_repo_checksums.return_value = result_obj
+            result = runner.invoke(app, ["cache", "verify", repo_id])
+
+        assert result.exit_code == 0
+        stdout = result.stdout
+        normalized_stdout = stdout.replace("\\", "/")
+        expected_path_str = verified_path.as_posix()
+        assert f"✅ Verified 1 file(s) for 'user/model' (model) in {expected_path_str}" in normalized_stdout
+        assert "  All checksums match." in stdout
+        get_api_mock.assert_called_once()
+        api.verify_repo_checksums.assert_called_once_with(
+            repo_id=repo_id,
+            repo_type="model",
+            revision=None,
+            cache_dir=None,
+            local_dir=None,
+            token=None,
+        )
+
+    def test_verify_reports_mismatch(self, runner: CliRunner) -> None:
+        repo_id = "user/model"
+        result_obj = FolderVerification(
+            revision="main",
+            checked_count=1,
+            mismatches=[{"path": "pytorch_model.bin", "expected": "dead", "actual": "beef", "algorithm": "sha256"}],
+            missing_paths=[],
+            extra_paths=[],
+            verified_path=Path("/tmp/cache/user/model"),
+        )
+
+        with patch("huggingface_hub.cli.cache.get_hf_api") as get_api_mock:
+            api = get_api_mock.return_value
+            api.verify_repo_checksums.return_value = result_obj
+            result = runner.invoke(app, ["cache", "verify", repo_id])
+
+        assert result.exit_code == 1
+        assert "Checksum verification failed" in result.stdout
+        assert "pytorch_model.bin" in result.stdout
+        assert "expected" in result.stdout
+        assert "Verification failed for 'user/model' (model)" in result.stdout
+        assert "Revision: main" in result.stdout
+
+    def test_verify_reports_missing_local_file(self, runner: CliRunner) -> None:
+        commit_hash = "4" * 40
+        repo_id = "user/model"
+        file_name = "config.json"
+
+        with SoftTemporaryDirectory() as tmp_dir:
+            base = Path(tmp_dir)
+            snapshot_path = base / "snapshots" / commit_hash
+            snapshot_path.mkdir(parents=True)
+
+            blob_dir = base / "blobs"
+            blob_dir.mkdir()
+
+            blob_path = blob_dir / ("a" * 64)
+            blob_path.write_bytes(b"hello")
+
+            file_path = snapshot_path / file_name
+            file_path.touch()
+
+            file_info = CachedFileInfo(
+                file_name=file_name,
+                file_path=file_path,
+                blob_path=blob_path,
+                size_on_disk=blob_path.stat().st_size,
+                blob_last_accessed=0.0,
+                blob_last_modified=0.0,
+            )
+            revision = CachedRevisionInfo(
+                commit_hash=commit_hash,
+                snapshot_path=snapshot_path,
+                size_on_disk=blob_path.stat().st_size,
+                files=frozenset({file_info}),
+                refs=frozenset({"main"}),
+                last_modified=0.0,
+            )
+            repo = CachedRepoInfo(
+                repo_id=repo_id,
+                repo_type="model",
+                repo_path=base,
+                size_on_disk=blob_path.stat().st_size,
+                nb_files=1,
+                revisions=frozenset({revision}),
+                last_accessed=0.0,
+                last_modified=0.0,
+            )
+            hf_cache_info = HFCacheInfo(
+                size_on_disk=blob_path.stat().st_size,
+                repos=frozenset({repo}),
+                warnings=[],
+            )
+
+            with (
+                patch("huggingface_hub.cli.cache.scan_cache_dir", return_value=hf_cache_info),
+                patch("huggingface_hub.cli.cache.get_hf_api") as get_api_mock,
+            ):
+                api = get_api_mock.return_value
+                api.list_repo_tree.return_value = [
+                    SimpleNamespace(path=file_name, blob_id="unused", lfs=None),
+                    SimpleNamespace(
+                        path="missing.txt",
+                        blob_id="blobid",
+                        lfs=None,
+                    ),
+                ]
+                result = runner.invoke(app, ["cache", "verify", repo.cache_id])
+
+        assert result.exit_code == 1
+        assert "missing locally" in result.stdout
+        assert "Verification failed for" in result.stdout
+        assert "Revision:" in result.stdout
+
 
 class TestUploadCommand:
     def test_upload_basic(self, runner: CliRunner) -> None:
@@ -1105,6 +1265,235 @@ class TestRepoDeleteCommand:
         )
 
 
+class TestInferenceEndpointsCommands:
+    def test_list(self, runner: CliRunner) -> None:
+        endpoint = Mock(raw={"name": "demo"})
+        with patch("huggingface_hub.cli.inference_endpoints.get_hf_api") as api_cls:
+            api = api_cls.return_value
+            api.list_inference_endpoints.return_value = [endpoint]
+            result = runner.invoke(app, ["endpoints", "ls"])
+        assert result.exit_code == 0
+        api_cls.assert_called_once_with(token=None)
+        api.list_inference_endpoints.assert_called_once_with(namespace=None, token=None)
+        assert '"items"' in result.stdout
+        assert '"name": "demo"' in result.stdout
+
+    def test_inference_endpoints_alias(self, runner: CliRunner) -> None:
+        endpoint = Mock(raw={"name": "alias"})
+        with patch("huggingface_hub.cli.inference_endpoints.get_hf_api") as api_cls:
+            api = api_cls.return_value
+            api.list_inference_endpoints.return_value = [endpoint]
+            result = runner.invoke(app, ["endpoints", "ls"])
+        assert result.exit_code == 0
+        api_cls.assert_called_once_with(token=None)
+        api.list_inference_endpoints.assert_called_once_with(namespace=None, token=None)
+        assert '"name": "alias"' in result.stdout
+
+    def test_deploy_from_hub(self, runner: CliRunner) -> None:
+        endpoint = Mock(raw={"name": "hub"})
+        with patch("huggingface_hub.cli.inference_endpoints.get_hf_api") as api_cls:
+            api = api_cls.return_value
+            api.create_inference_endpoint.return_value = endpoint
+            result = runner.invoke(
+                app,
+                [
+                    "endpoints",
+                    "deploy",
+                    "my-endpoint",
+                    "--repo",
+                    "my-repo",
+                    "--framework",
+                    "custom",
+                    "--accelerator",
+                    "cpu",
+                    "--instance-size",
+                    "x4",
+                    "--instance-type",
+                    "standard",
+                    "--region",
+                    "us-east-1",
+                    "--vendor",
+                    "aws",
+                ],
+            )
+        assert result.exit_code == 0
+        api_cls.assert_called_once_with(token=None)
+        api.create_inference_endpoint.assert_called_once_with(
+            name="my-endpoint",
+            repository="my-repo",
+            framework="custom",
+            accelerator="cpu",
+            instance_size="x4",
+            instance_type="standard",
+            region="us-east-1",
+            vendor="aws",
+            namespace=None,
+            token=None,
+            task=None,
+        )
+        assert '"name": "hub"' in result.stdout
+
+    def test_deploy_from_catalog(self, runner: CliRunner) -> None:
+        endpoint = Mock(raw={"name": "catalog"})
+        with patch("huggingface_hub.cli.inference_endpoints.get_hf_api") as api_cls:
+            api = api_cls.return_value
+            api.create_inference_endpoint_from_catalog.return_value = endpoint
+            result = runner.invoke(
+                app,
+                [
+                    "endpoints",
+                    "catalog",
+                    "deploy",
+                    "--repo",
+                    "catalog/model",
+                ],
+            )
+        assert result.exit_code == 0
+        api_cls.assert_called_once_with(token=None)
+        api.create_inference_endpoint_from_catalog.assert_called_once_with(
+            repo_id="catalog/model",
+            name=None,
+            namespace=None,
+            token=None,
+        )
+        assert '"name": "catalog"' in result.stdout
+
+    def test_describe(self, runner: CliRunner) -> None:
+        endpoint = Mock(raw={"name": "describe"})
+        with patch("huggingface_hub.cli.inference_endpoints.get_hf_api") as api_cls:
+            api = api_cls.return_value
+            api.get_inference_endpoint.return_value = endpoint
+            result = runner.invoke(app, ["endpoints", "describe", "my-endpoint"])
+        assert result.exit_code == 0
+        api_cls.assert_called_once_with(token=None)
+        api.get_inference_endpoint.assert_called_once_with(name="my-endpoint", namespace=None, token=None)
+        assert '"name": "describe"' in result.stdout
+
+    def test_update(self, runner: CliRunner) -> None:
+        endpoint = Mock(raw={"name": "updated"})
+        with patch("huggingface_hub.cli.inference_endpoints.get_hf_api") as api_cls:
+            api = api_cls.return_value
+            api.update_inference_endpoint.return_value = endpoint
+            result = runner.invoke(
+                app,
+                [
+                    "endpoints",
+                    "update",
+                    "my-endpoint",
+                    "--repo",
+                    "my-repo",
+                    "--accelerator",
+                    "gpu",
+                    "--instance-size",
+                    "x4",
+                ],
+            )
+        assert result.exit_code == 0
+        api_cls.assert_called_once_with(token=None)
+        api.update_inference_endpoint.assert_called_once_with(
+            name="my-endpoint",
+            namespace=None,
+            repository="my-repo",
+            framework=None,
+            revision=None,
+            task=None,
+            accelerator="gpu",
+            instance_size="x4",
+            instance_type=None,
+            min_replica=None,
+            max_replica=None,
+            scale_to_zero_timeout=None,
+            token=None,
+        )
+        assert '"name": "updated"' in result.stdout
+
+    def test_delete(self, runner: CliRunner) -> None:
+        with patch("huggingface_hub.cli.inference_endpoints.get_hf_api") as api_cls:
+            api = api_cls.return_value
+            result = runner.invoke(app, ["endpoints", "delete", "my-endpoint", "--yes"])
+        assert result.exit_code == 0
+        api_cls.assert_called_once_with(token=None)
+        api.delete_inference_endpoint.assert_called_once_with(name="my-endpoint", namespace=None, token=None)
+        assert "Deleted 'my-endpoint'." in result.stdout
+
+    def test_pause(self, runner: CliRunner) -> None:
+        endpoint = Mock(raw={"name": "paused"})
+        with patch("huggingface_hub.cli.inference_endpoints.get_hf_api") as api_cls:
+            api = api_cls.return_value
+            api.pause_inference_endpoint.return_value = endpoint
+            result = runner.invoke(app, ["endpoints", "pause", "my-endpoint"])
+        assert result.exit_code == 0
+        api_cls.assert_called_once_with(token=None)
+        api.pause_inference_endpoint.assert_called_once_with(name="my-endpoint", namespace=None, token=None)
+        assert '"name": "paused"' in result.stdout
+
+    def test_resume(self, runner: CliRunner) -> None:
+        endpoint = Mock(raw={"name": "resumed"})
+        with patch("huggingface_hub.cli.inference_endpoints.get_hf_api") as api_cls:
+            api = api_cls.return_value
+            api.resume_inference_endpoint.return_value = endpoint
+            result = runner.invoke(app, ["endpoints", "resume", "my-endpoint"])
+        assert result.exit_code == 0
+        api_cls.assert_called_once_with(token=None)
+        api.resume_inference_endpoint.assert_called_once_with(
+            name="my-endpoint",
+            namespace=None,
+            token=None,
+            running_ok=True,
+        )
+        assert '"name": "resumed"' in result.stdout
+
+    def test_resume_fail_if_already_running(self, runner: CliRunner) -> None:
+        endpoint = Mock(raw={"name": "resumed"})
+        with patch("huggingface_hub.cli.inference_endpoints.get_hf_api") as api_cls:
+            api = api_cls.return_value
+            api.resume_inference_endpoint.return_value = endpoint
+            result = runner.invoke(
+                app,
+                [
+                    "endpoints",
+                    "resume",
+                    "my-endpoint",
+                    "--fail-if-already-running",
+                ],
+            )
+        assert result.exit_code == 0
+        api_cls.assert_called_once_with(token=None)
+        api.resume_inference_endpoint.assert_called_once_with(
+            name="my-endpoint",
+            namespace=None,
+            token=None,
+            running_ok=False,
+        )
+        assert '"name": "resumed"' in result.stdout
+
+    def test_scale_to_zero(self, runner: CliRunner) -> None:
+        endpoint = Mock(raw={"name": "zero"})
+        with patch("huggingface_hub.cli.inference_endpoints.get_hf_api") as api_cls:
+            api = api_cls.return_value
+            api.scale_to_zero_inference_endpoint.return_value = endpoint
+            result = runner.invoke(app, ["endpoints", "scale-to-zero", "my-endpoint"])
+        assert result.exit_code == 0
+        api_cls.assert_called_once_with(token=None)
+        api.scale_to_zero_inference_endpoint.assert_called_once_with(
+            name="my-endpoint",
+            namespace=None,
+            token=None,
+        )
+        assert '"name": "zero"' in result.stdout
+
+    def test_list_catalog(self, runner: CliRunner) -> None:
+        with patch("huggingface_hub.cli.inference_endpoints.get_hf_api") as api_cls:
+            api = api_cls.return_value
+            api.list_inference_catalog.return_value = ["model"]
+            result = runner.invoke(app, ["endpoints", "catalog", "ls"])
+        assert result.exit_code == 0
+        api_cls.assert_called_once_with(token=None)
+        api.list_inference_catalog.assert_called_once_with(token=None)
+        assert '"models"' in result.stdout
+        assert '"model"' in result.stdout
+
+
 @contextmanager
 def tmp_current_directory() -> Generator[str, None, None]:
     with SoftTemporaryDirectory() as tmp_dir:
